@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # This script can be used to intercept and forward the inverter log messages.
 # The script has been tested with python versions 2.7 and 3.8
@@ -11,6 +11,7 @@
 #
 
 import socket
+import socketserver
 import argparse
 import configparser
 import threading
@@ -24,16 +25,16 @@ import logging
 import datetime
 import time
 
-__version__ = '1.0.3'
-listenaddress = b'127.0.0.1'                       # <<<< change this to your ip address >>>>
+__version__ = '1.1.5'
+listenaddress = b'127.0.0.1'                       # Default listenaddress
 listenport = 10004                                 # Make sure your firewall enables you listening at this port
 # There is no need to change this if this proxy must log your data directly to the Omnik/SolarmanPV servers
-omnikloggerpublicaddress = b'176.58.117.69'        # There is no need to change this if this proxy
+omnikloggerpublicaddress = b'176.58.117.69'        # If you have an aditional omniklogger, change the ip to enlarge the chain
 omnikloggerdestport = 10004                        # This is the port the omnik/SolarmanPV datalogger server listens to
 STATUS_ON = 'ON'
 STATUS_OFF = 'OFF'
 CHECK_STATUS_INTERVAL = 60
-INVERTER_MAX_IDLE_TIME = 10
+INVERTER_MAX_IDLE_TIME = 6
 
 global stopflag
 
@@ -48,141 +49,82 @@ def signal_handler(signal, frame):
 class ProxyServer(threading.Thread):
 
     def __init__(self, args=[], kwargs={}):
-        self.stopsignal = 0
-        self.sockwait = threading.Event()
-
         threading.Thread.__init__(self)
-        self.data = None
-        self.connection = None
-        self.sock = None
         if args.mqtt_host:
-            self.mqttfw = mqtt(args)
-        else:
-            self.mqttfw = None
-        self.status = {}
-        self.lastupdate = {}
+            logging.info('Enabling MQTT forward to {0}'.format(args.mqtt_host))
+            RequestHandler.mqttfw = mqtt(args)
+            # Check every 60 sec if there is a valid update status aged less than 30 minutes
+            self.statustimer = threading.Timer(CHECK_STATUS_INTERVAL, self.check_status)
+            self.statustimer.start()
 
-        # Check every 60 sec if there is a valid update status aged less than 30 minutes
-        self.statustimer = threading.Timer(CHECK_STATUS_INTERVAL, self.check_status)
-        self.statustimer.start()
+        # Create tcp server
+        self.tcpServer = socketserver.TCPServer((args.listenaddress, args.listenport), RequestHandler)
 
     def check_status(self):
-        for serial in self.lastupdate:
-            if self.lastupdate[serial] + datetime.timedelta(minutes=INVERTER_MAX_IDLE_TIME) < datetime.datetime.now():
-                self.status[serial] = STATUS_OFF
-                self.mqttfw.mqttforward('',
-                                        serial, self.status[serial])
+        for serial in RequestHandler.lastupdate:
+            if RequestHandler.lastupdate[serial] + datetime.timedelta(minutes=INVERTER_MAX_IDLE_TIME) < \
+                    datetime.datetime.now():
+                RequestHandler.status[serial] = STATUS_OFF
+                RequestHandler.mqttfw.mqttforward('',
+                                                  serial, RequestHandler.status[serial])
         # Restart timer
         self.statustimer = threading.Timer(CHECK_STATUS_INTERVAL, self.check_status)
         self.statustimer.start()
 
     def cancel(self):
         # Stop the loop
-        self.stopsignal = 1
-        try:
+        self.tcpServer.shutdown()
+        if RequestHandler.mqttfw:
             self.statustimer.cancel()
-        except Exception as e:
-            logging.debug('Cannot stop status timer. Error: {0}'.format(e))
-        try:
-            # Work-a-round to close listening socket
-            socket.socket(socket.AF_INET,
-                          socket.SOCK_STREAM).connect(self.server_address)
-            self.sock.close()
-        except Exception as e:
-            logging.debug('No open listening socket to close. Error: {0}'.format(e))
-        if self.mqttfw:
-            self.mqttfw.close()
-        self.sockwait.clear()
-
-    def sockaccept(self):
-        # Wait for connection
-        try:
-            self.connection, self.client_address = self.sock.accept()
-            if not self.stopsignal:
-                logging.debug('client connected: {0}'.format(self.client_address))
-                # Process received data
-                self.data = self.connection.recv(1024)
-        except Exception as e:
-            logging.debug('Unable to receive message. Error: {0}'.format(e))
-        self.sockwait.set()
+            RequestHandler.mqttfw.close()
 
     def run(self):
         # Try to run TCP server
-        # Create a TCP/IP socket to listen
-        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        logging.info('Start in Omnik proxy server. Listening to {0}:{1}'.format(args.listenaddress, args.listenport))
+        self.tcpServer.serve_forever()
 
-        # Bind the socket to the address given on the command line
-        self.server_address = (args.listenaddress, args.listenport)
-        try:
-            self.sock.bind(self.server_address)
-            self.sock.settimeout(40)
-        except Exception as e:
-            logging.critical('Not able to open socket for listening, exiting! Error: {0}'.format(e))
-            self.cancel()
-            return
 
-        logging.info('starting up on %s port %s' % self.sock.getsockname())
-        # Accept max 1 connection
-        self.sock.listen(1)
+class RequestHandler(socketserver.BaseRequestHandler):
 
-        # Main loop
-        while not self.stopsignal:
-            logging.debug('waiting for a connection')
+    mqttfw = None
+    status = {}
+    lastupdate = {}
 
-            # Initialize event for handling accept socket
-            self.sockwait = threading.Event()
-            # Initialize data var
-            self.data = None
-            self.serial = None
+    def handle(self):
+        msg = self.request.recv(1024)
 
-            acceptthread = threading.Thread(target=self.sockaccept)
-            # Wait for accept
-            acceptthread.start()
-            # The following call will block the thead
-            self.sockwait.wait()
-            if self.connection:
-                if self.data and len(self.data) == 128:
-                    self._processmsg()
-                try:
-                    self.connection.close()
-                    logging.debug('connection closed')
-                except Exception as e:
-                    logging.debug('closing connection failed: {0}'.format(e))
+        if len(msg) >= 99:
+            self._processmsg(msg)
 
-    def _processmsg(self):
-        rawserial = str(self.data[15:31].decode())
+    def _processmsg(self, msg):
+        rawserial = str(msg[15:31].decode())
         # obj dLog transforms de raw data from the photovoltaic Systems converter
         valid = False
         if rawserial in args.serialnumber:
-            self.status[rawserial] = STATUS_ON
-            self.lastupdate[rawserial] = datetime.datetime.now()
+            # TODO: Dit gaat niet werken zo
+            RequestHandler.status[rawserial] = STATUS_ON
+            RequestHandler.lastupdate[rawserial] = datetime.datetime.now()
             valid = True
         if valid:
             # Process data
             logging.info("Processing message for inverter '{0}'".format(rawserial))
-            self.forwardstate(rawserial, self.status[rawserial])
-            if self.connection:
-                try:
-                    self.connection.sendall(b'')
-                except Exception as e:
-                    logging.debug('Unabled to reply to log request: {0}'.format(e))
-                # give replay if connection is not closed yet
+            self.forwardstate(msg, rawserial, self.status[rawserial])
         else:
-            logging.warning("Received incorrect data, ignoring connection!")
+            logging.warning("Ignoring received data!")
 
-    def forwardstate(self, serial, status):
+    def forwardstate(self, msg, serial, status):
         # Forward data to datalogger and MQTT
         # TODO
         # Forward logger message to MTTQ if host is defined
-        if self.mqttfw:
-            self.mqttfw.mqttforward(json.dumps(str(binascii.b2a_base64(self.data), encoding='ascii')),
-                                    serial, status)
+        if RequestHandler.mqttfw:
+            RequestHandler.mqttfw.mqttforward(json.dumps(str(binascii.b2a_base64(msg), encoding='ascii')),
+                                              serial, status)
         # Forward data to proxy or Omnik datalogger
         if args.omniklogger:
             self.fwthread = tcpforward()
-            self.fwthread.forward(self.data)
+            self.fwthread.forward(msg)
             self.fwthread.join(60)
-            if self.fwthread.isAlive():
+            if self.fwthread.is_alive():
                 # Kill the thread if it is still blocking
                 logging.warning("Waiting for forward connection takes a long time...")
                 self.fwthread.join()
@@ -201,9 +143,8 @@ class tcpforward(threading.Thread):
         self.forwardsock.settimeout(90)
         try:
             self.forwardsock.connect(loggeraddress)
-            logging.info('{0} Forwarding to omnik logger "{1}"'.format(datetime.datetime.now(), args.omniklogger))
             self.forwardsock.sendall(self.data)
-            logging.info('Forwarding succesful.')
+            logging.info('{0} Forwarded to "{1}"'.format(datetime.datetime.now(), args.omniklogger))
             self.forwardsock.close()
         except Exception as e:
             logging.warning("Error forwarding: {0}".format(e))
@@ -239,6 +180,9 @@ class mqtt(object):
         self.device_name = args.mqtt_device_name
         self.logger_sensor_name = args.mqtt_logger_sensor_name
 
+        # lock
+        self.lock = threading.Condition(threading.Lock())
+
     def close(self):
         self.mqtt_client.disconnect()
 
@@ -301,41 +245,49 @@ class mqtt(object):
             # publish config
             if self.mqtt_client.publish(topics['config'][entity], json.dumps(config_pl[entity]), retain=self.mqtt_retain):
                 logging.debug("Publishing config {0} successful.".format(entity))
+                return True
             else:
                 logging.warning("Publishing config {0} failed!".format(entity))
+                return False
         except Exception as e:
             logging.error("Unhandled error publishing config for entity {0}: {1}".format(entity, e))
+            return False
 
     def _publish_attributes(self, topics, attr_pl):
         try:
             # publish attributes
             if self.mqtt_client.publish(topics['attr'], json.dumps(attr_pl), retain=self.mqtt_retain):
                 logging.debug("Publishing attributes successful.")
+                return True
             else:
                 logging.warning("Publishing attributes failed!")
+                return False
         except Exception as e:
             logging.error("Unhandled error publishing attributes: {0}".format(e))
+            return False
 
     def _publish_state(self, topics, value_pl):
         try:
             # publish state
             if self.mqtt_client.publish(topics['state'], json.dumps(value_pl), retain=self.mqtt_retain):
                 logging.debug("Publishing state {0} successful.".format(json.dumps(value_pl)))
+                return True
             else:
                 logging.warning("Publishing state {0} failed!".format(json.dumps(value_pl)))
+                return False
         except Exception as e:
             logging.error("Unhandled error publishing states: {0}".format(e))
+            return False
 
     def mqttforward(self, data, serial, status):
         # Decode data: base64.b64decode(json.loads(d, encoding='UTF-8'))
+        self.lock.acquire()
         self.data = data
         self.serial = serial
         self.status = status
         self.reporttime = datetime.datetime.now()
         # Get topics
         topics = self._topics()
-
-        logging.debug("{0} Forwarding message to MQTT service".format(self.reporttime))
         # publish attributes
         attr_pl = self._attribute_payload()
         self._publish_attributes(topics, attr_pl)
@@ -347,7 +299,10 @@ class mqtt(object):
 
         # publish state
         value_pl = self._value_payload()
-        self._publish_state(topics, value_pl)
+        if self._publish_state(topics, value_pl):
+            logging.info('{0} Message forwarded to MQTT service at "{1}"'.format(datetime.datetime.now(), args.mqtt_host))
+
+        self.lock.release()
 
     def _mqtt_on_connect(self, client, userdata, flags, rc):
         if rc == 0:
@@ -365,7 +320,7 @@ def main(args):
     proxy = ProxyServer(args)
     proxy.start()
     try:
-        while not (proxy.stopsignal or stopflag):
+        while not stopflag and proxy.is_alive():
             proxy.join(1)
     except KeyboardInterrupt:
         pass
@@ -379,21 +334,20 @@ if __name__ == '__main__':
     home = os.path.expanduser('~')
 
     parser = argparse.ArgumentParser()
-
-    parser.add_argument('--serialnumber', default=None, nargs='+', required=True,
-                        help='The serial number(s) of your inverter')
     parser.add_argument('--config', default=os.path.join(home, '.omnik/config.ini'),
                         help='The config file')
-    parser.add_argument('--loglevel', default='INFO',
+    parser.add_argument('--serialnumber', default=None, nargs='+',
+                        help='The serial number(s) of your inverter (required)')
+    parser.add_argument('--loglevel', default=None,
                         help='The basic loglevel [DEBUG, INFO, WARNING, ERROR, CRITICAL]')
-    parser.add_argument('--listenaddress', default=listenaddress,
+    parser.add_argument('--listenaddress', default=None,
                         help='A local available address to listen to')
-    parser.add_argument('--listenport', default=listenport, type=int,
+    parser.add_argument('--listenport', default=None, type=int,
                         help='The local port to listen to')
     parser.add_argument('--omniklogger', default=None,
                         help='Forward to an address omnik/SolarmanPV datalogger server listens to. '
                              'Set this to {0} as final forwarder.'.format(omnikloggerpublicaddress))
-    parser.add_argument('--omnikloggerport', default=omnikloggerdestport,
+    parser.add_argument('--omnikloggerport', default=None, type=int,
                         help='The port the omnik/SolarmanPV datalogger server listens to')
     parser.add_argument('--mqtt_host', default=None,
                         help='The mqtt host to forward processed data to. Config overrides.')
@@ -402,7 +356,7 @@ if __name__ == '__main__':
     parser.add_argument('--mqtt_retain', default=True, type=bool,
                         help='The mqtt data message retains. Config overrides.')
     parser.add_argument('--mqtt_discovery_prefix', default='homeassistant',
-                        help='The mqtt topoc prefix (used for MQTT auto discovery). Config overrides.')
+                        help='The mqtt topic prefix (used for MQTT auto discovery). Config overrides.')
     parser.add_argument('--mqtt_client_name_prefix', default='ha-mqtt-omnikloggerproxy',
                         help='The port the omnik/SolarmanPV datalogger server listens to. Config overrides.')
     parser.add_argument('--mqtt_username', default=None,
@@ -422,19 +376,35 @@ if __name__ == '__main__':
         "ERROR": logging.ERROR,
         "CRITICAL": logging.CRITICAL
         }
-    logging.basicConfig(level=loglevel[args.loglevel], format=FORMAT)
 
     if args.config:
         c = configparser.ConfigParser(converters={'list': lambda x: [i.strip() for i in x.split(',')]})
         c.read([args.config], encoding='utf-8')
-        args.mqtt_host = c.get('mqtt', 'host', fallback=args.mqtt_host)
-        args.mqtt_port = c.get('mqtt', 'port', fallback=args.mqtt_port)
-        args.mqtt_retain = True if c.get('mqtt', 'retain', fallback='true') == 'true' else args.mqtt_retain
-        args.mqtt_discovery_prefix = c.get('mqtt', 'discovery_prefix ', fallback=args.mqtt_discovery_prefix)
-        args.mqtt_client_name_prefix = c.get('mqtt', 'client_name_prefix', fallback=args.mqtt_client_name_prefix)
-        args.mqtt_username = c.get('mqtt', 'username', fallback=args.mqtt_username)
-        args.mqtt_password = c.get('mqtt', 'password', fallback=args.mqtt_password)
-        args.mqtt_device_name = c.get('mqtt', 'device_name', fallback=args.mqtt_device_name)
-        args.logger_sensor_name = c.get('mqtt', 'logger_sensor_name', fallback=args.mqtt_logger_sensor_name)
+        args.mqtt_host = c.get('output.mqtt', 'host', fallback=args.mqtt_host)
+        args.mqtt_port = c.get('output.mqtt', 'port', fallback=args.mqtt_port)
+        args.mqtt_retain = True if c.get('output.mqtt', 'retain', fallback='true') == 'true' else args.mqtt_retain
+        args.mqtt_discovery_prefix = c.get('output.mqtt', 'discovery_prefix ', fallback=args.mqtt_discovery_prefix)
+        args.mqtt_client_name_prefix = c.get('output.mqtt', 'client_name_prefix', fallback=args.mqtt_client_name_prefix)
+        args.mqtt_username = c.get('output.mqtt', 'username', fallback=args.mqtt_username)
+        args.mqtt_password = c.get('output.mqtt', 'password', fallback=args.mqtt_password)
+        args.mqtt_device_name = c.get('output.mqtt', 'device_name', fallback=args.mqtt_device_name)
+        args.logger_sensor_name = c.get('output.mqtt', 'logger_sensor_name', fallback=args.mqtt_logger_sensor_name)
 
+        # TODO: Try to get from config from no arguments are given. Handle required parameters and defaults
+        if not args.serialnumber:
+            args.serialnumber = c.getlist('proxy', 'serialnumber', fallback=[])
+        if not args.loglevel:
+            args.loglevel = c.get('proxy', 'loglevel', fallback='INFO')
+        if not args.listenaddress:
+            args.listenaddress = c.get('proxy', 'listenaddress', fallback=listenaddress)
+        if not args.listenport:
+            args.listenport = c.getint('proxy', 'listenport', fallback=listenport)
+        if not args.omniklogger:
+            args.omniklogger = c.get('proxy', 'omniklogger', fallback=omnikloggerpublicaddress)
+        if not args.omnikloggerport:
+            args.omnikloggerport = c.getint('proxy', 'omnikloggerport', fallback=omnikloggerdestport)
+    if not args.serialnumber:
+        parser.print_help()
+        os.sys.exit(1)
+    logging.basicConfig(level=loglevel[args.loglevel], format=FORMAT)
     main(args)
